/* PC KEYBOARD Interface for AVR
 ***********************
 * DATA on PORTD BIT 5 *
 * SCK  on INTERRUPT 2 *
 ***********************
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "keyboard.h"
#include "framebuffer.h"
#include "uart.h"

typedef unsigned char scan_code[2]; // for scancodes.h
#include "scancodes.h"


volatile uint8_t kb_buffer[KB_BUFF_SIZE];
volatile uint8_t kb_bufpos;
unsigned char is_up=0,shift=0,funckey=0,altgr=0;
unsigned char edge, bitcount;                // 0 = neg.  1 = pos.


void kbInit(void)  // external interupt Enable and stuff
{
	kb_bufpos = 0;
	shift= 0;

	EIMSK = 2;
    EICRA = 8;                                // INT0 interrupt on falling edge
	 
    edge = 0;                                // 0 = falling edge  1 = rising edge
    bitcount = 11;

	// Dataport auf Eingang
	KB_DDR  &= ~( 1 << KB_DATA );

	// Dataport Pullup activieren
	KB_PORT |= ( 1 << KB_DATA );

	// Datentransver anhalten
	kbStopTransfer();

}

void kbStopTransfer ()
{
	// Clock auf Ausgang
	KB_DDR  |=  ( 1 << KB_CLOCK );
	
	// Clock auf Low- Ziehen, damit angehalten wird
	KB_PORT &= ~( 1 << KB_CLOCK );
}

void kbStartTransfer ()
{

	bitcount = 11;
	edge = 0;

	// Clock auf Hight- Ziehen, damit bertragung starten kann
	//KB_PORT |=  ( 1 << KB_CLOCK );

	// Clock auf Eingang
	KB_DDR  &= ~( 1 << KB_CLOCK );
}


ISR ( INT1_vect )
{
    static unsigned char data;                // Holds the received scan code

    if (!edge)                                // Routine entered at falling edge
    {
        if(bitcount < 11 && bitcount > 2)    // Bit 3 to 10 is data. Parity bit,
        {                                    // start and stop bits are ignored.
            data = (data >> 1);
            if(KB_IN & (1 << KB_DATA))
                data = data | 0x80;            // Store a '1'
        }

        EICRA = 8+4;                            // Set interrupt on rising edge
        edge = 1;
        
    } 
	else 
	{                                // Routine entered at rising edge

        if(--bitcount == 0)                    // All bits received
        {
			decode (data);
            bitcount = 11;
        }
		EICRA = 8;                            // Set interrupt on falling edge
		edge = 0;
    }
}


void put_kbbuff(unsigned char c)
{
	/*
	if(c == '\n')
		cursor_newline();
	else
	{
		fb_putchar(c);
		cursor_advance();
	}
	*/
	uart_putchar(c);
}


uint8_t decode(unsigned char sc)
{
	unsigned char i;
	uint8_t decode_complete = 0;

    if (is_up)    {
        if(sc!=0xE0)
        {
        	is_up = 0;                         // Two 0xF0 in a row not allowed
       	}
        funckey=0;
        switch (sc)
        {
          case 0x12 : shift = 0; break;             // Left SHIFT
          case 0x59 : shift = 0; break;             // Right SHIFT
          case 0x11 : altgr = 0; break;
    //      case 0x76 : break;                      // ESC
        }
    }else
    {
        switch (sc)
        {
          case 0xF0 : is_up = 1; break;      // The up-key identifier
          case 0x12 : shift = 1; break;      // Left SHIFT
          case 0x59 : shift = 1; break;      // Right SHIFT
          case 0x58 : shift = 1; break;      // Caps Lock
          case 0xE0 : funckey=1; break;
          case 0x11 : altgr = 1; break;
          
          default:
          { 
            if (funckey)
            {
            	put_kbbuff(0);
            	put_kbbuff(sc);
            	funckey=0;
            } else if (altgr)
            {
                for(i = 0; pgm_read_byte(&alt_gr[i][0])!=sc && pgm_read_byte(&alt_gr[i][0]); i++);
                if (pgm_read_byte(&alt_gr[i][0]) == sc) {
                    put_kbbuff(pgm_read_byte(&alt_gr[i][1]));
                }
            }
            else if(!shift)                    // If shift not pressed,
            {                             // do a table look-up
                for(i = 0; pgm_read_byte(&unshifted[i][0])!=sc && pgm_read_byte(&unshifted[i][0]); i++);
                if (pgm_read_byte(&unshifted[i][0]) == sc) {
                    put_kbbuff(pgm_read_byte(&unshifted[i][1]));
                }  else {put_kbbuff(0); put_kbbuff(sc);}
            } else {                    // If shift pressed
                for(i = 0; pgm_read_byte(&shifted[i][0])!=sc && pgm_read_byte(&shifted[i][0]); i++);
                if (pgm_read_byte(&shifted[i][0]) == sc) {
                    put_kbbuff(pgm_read_byte(&shifted[i][1]));
                }  else {put_kbbuff(0); put_kbbuff(sc);}
            }
			decode_complete = 1;
          }break;  
        } 
        
    }
	if(decode_complete) return 1;
	else return 0;
    
}

